/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation.
 * Licensed Material - Property of IBM. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ui.examples.navigator;

//import java.io.BufferedReader;
//import java.io.IOException;
//import java.io.InputStream;
//import java.io.StringReader;
//import java.util.ArrayList;
//import java.util.Enumeration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
//import java.util.List;
import java.util.Map;
//import java.util.Properties;


import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.progress.UIJob;

/**
 * Provides the properties contained in a *.properties file as children of that
 * file in a Common Navigator.  
 * @since 3.2 
 */
public class PropertiesContentProvider implements ITreeContentProvider,
		IResourceChangeListener, IResourceDeltaVisitor {
  
	private static final Object[] NO_CHILDREN = new Object[0];

//	private static final Object PROPERTIES_EXT = "properties"; //$NON-NLS-1$
	
	private static final Object STANDORTS_AND_SERVERS = "properties"; //$NON-NLS-1$

	private final Map/*<IFile, PropertiesTreeData[]>*/ cachedModelMap = new HashMap();

	private StructuredViewer viewer;
	
	/**
	 * Create the PropertiesContentProvider instance.
	 * 
	 * Adds the content provider as a resource change listener to track changes on disk.
	 *
	 */
	public PropertiesContentProvider() {
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE);
		updateModel1();
	}

	/**
	 * Return the model elements for a *.properties IFile or
	 * NO_CHILDREN for otherwise.
	 */
	public Object[] getChildren(Object parentElement) {  
		Object[] children = null;
		if (parentElement instanceof Server) { 
			children = NO_CHILDREN;
//		} else 
//			if(parentElement instanceof Element) {
//			children = ((Element)parentElement).elements().toArray();
		} else 
			if(parentElement instanceof Location) {
			List list = ((Location)parentElement).getMChildLocations();
			list.addAll(  ((Location)parentElement).getMServerList() );
			children = list.toArray();
//		} else if(parentElement instanceof IFile) {
//			IFile modelFile = (IFile) parentElement;
//			if(STANDORTS_AND_SERVERS.equals(modelFile.getFileExtension())) {
//				 List elements = updateModel(modelFile).elements();
//				children = elements.toArray();
//			}
			
			} else if(parentElement instanceof IResource) {
//				IResource modelFile = (IResource) parentElement;
//				if(STANDORTS_AND_SERVERS.equals(modelFile.getFileExtension())) {
				children = updateModel1().toArray();					
			}

		   
		return children != null ? children : NO_CHILDREN;
	}  

	
	
	private synchronized List updateModel1() { 
		
		ArrayList list = new ArrayList();
		
		ArrayList serverlist = new ArrayList();
		serverlist.add(new Server("mac","ihu","cc"));  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
		
		ArrayList list1 = new ArrayList();				
		
		list1.add(new Location("1","2","3", "1","2","3", 1,new ArrayList(),serverlist));   //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
		
		list.add(new Location("4","5","6", "4","5","6", 1,list1,serverlist));   //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$

		return list; 
	}

	public Object getParent(Object element) {

//		if( element instanceof Element ){
//			return ((Element)element).getParent();
//		}
		return null;
	}

	public boolean hasChildren(Object element) {		

//			if(element instanceof Element) {
//			return ((Element)element).elements().size()>0;
//		} else if(element instanceof IResource) {
//			return STANDORTS_AND_SERVERS.equals(((IResource) element).getFileExtension());
//		}
//		return false;
		//weztodo:
		if((element instanceof Server) || ( element instanceof Location &&  ((Location)element).getMServerList().size()<=0  && ((Location)element).getMChildLocations().size()<=0 )) {
			return false;
		}
		return true;
	}

	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	public void dispose() {
		cachedModelMap.clear();
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this); 
	}

	public void inputChanged(Viewer aViewer, Object oldInput, Object newInput) {
		if (oldInput != null && !oldInput.equals(newInput))
			cachedModelMap.clear();
		viewer = (StructuredViewer) aViewer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {

		IResourceDelta delta = event.getDelta();
		try {
			delta.accept(this);
		} catch (CoreException e) { 
			e.printStackTrace();
		} 
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IResourceDeltaVisitor#visit(org.eclipse.core.resources.IResourceDelta)
	 */
	public boolean visit(IResourceDelta delta) {

		IResource source = delta.getResource();
		switch (source.getType()) {
		case IResource.ROOT:
			final IResource file = (IResource) source;
//			if (STANDORTS_AND_SERVERS.equals(file.getFileExtension())) {
				updateModel1();
				new UIJob("Update Properties Model in CommonViewer") {  //$NON-NLS-1$
					public IStatus runInUIThread(IProgressMonitor monitor) {
						if (viewer != null && !viewer.getControl().isDisposed())
							viewer.refresh(file);
						return Status.OK_STATUS;						
					}
				}.schedule();
//			}
			return false;			
		case IResource.PROJECT:

		case IResource.FOLDER:
			return true;
		case IResource.FILE:
			/*
			final IResource file = (IResource) source;
//			if (STANDORTS_AND_SERVERS.equals(file.getFileExtension())) {
				updateModel1();
				new UIJob("Update Properties Model in CommonViewer") {  //$NON-NLS-1$
					public IStatus runInUIThread(IProgressMonitor monitor) {
						if (viewer != null && !viewer.getControl().isDisposed())
							viewer.refresh(file);
						return Status.OK_STATUS;						
					}
				}.schedule();
//			}
			return false;
			*/
		}
		return false;
	} 
}
